*********************************************************************
*
* Your line routine.
* Every time you are given: A0.L : Chunky Screen (256x256x8)
*
* d0.w/d1.w  x1/y1
* d2.w/d3.w  x2/y2
* d4.b       color
*
* other registers are in undefined state !!
* you may trash all regs except a7
*
*********************************************************************

	CNOP	0,4
_line
c

;****************************************************************************
;**
;*       Speed-optimized chunky linedraw routine, A
;*
;*  This is one of my contributions to #amycoders linedrawing-compo. This
;*  routine will participate speed-optimizing competition. This code works
;*  with all 0x0 processors, with any cache setup.
;*
;*  This is a simple Bresenham's line algorithm implementation with separate
;*  optimized loops for x stepping.
;*
;*  Code size without rts: 156 bytes.
;*  Average time (dec): 41 (030@59.220MHz)
;*
;*  Coded by Harry "Piru" Sintonen <sintonen@jyu.fi>.
;*  This code is copyright  1997 by Harry Sintonen, and was written 18th-
;*  19th september 1997.
;*
;****************************************************************************
;*

;  IN: d0.w=x1, d1.w=y1, d2.w=x2, d3.w=y2, d4.b=color
;      a0=chunky buffer (256x256x8)
; OUT: line :) and d0-d3/d5-d6/a0 trashed

S_WIDTH2	EQU	8
S_WIDTH	EQU	1<<S_WIDTH2		256 (screen width)

; following is my quick "nop" longword align macro ;)
mcnop4tempb	; this must be longword aligned!
mcnop4	MACRO
mcnop4temps	SET	(*-mcnop4tempb)&2
	IFGT	(mcnop4temps)
	moveq	#0,d6
	ENDM

	moveq	#S_WIDTH>>6,d5	sy=S_WIDTH
	sub.w	d1,d3		d3=deltay
	bpl.b	.noyswap
	neg.l	d5		sy=-S_WIDTH
	neg.w	d3
.noyswap
	ext.l	d1
	asl.l	#6,d5
	lsl.l	#S_WIDTH2,d1
	or.w	d0,d1
	sub.w	d0,d2		d2=deltax
	add.l	d1,a0		add x1+y1*S_WIDTH to chunky screen
	bpl.b	.pos_dx
	neg.w	d2

	cmp.w	d3,d2
	bhs.b	.nx_walkx

	move.w	d3,d0		numpixels (deltay)
	neg.w	d3
	add.w	d2,d2		2*deltax
	move.w	d3,d1		d1=(2*deltax-deltay) -2*deltax = -deltay
	add.w	d3,d3		d3=2*(deltax-deltay) -2*deltax = -2*deltay

	mcnop4
.nx_yloop	move.b	d4,(a0)		put pixel
	add.w	d2,d1		d=d+2*deltax
	add.l	d5,a0		y=y+sy
	dbpl	d0,.nx_yloop
	add.w	d3,d1		d=d+2*(deltax-deltay)-2*deltax
	subq.l	#1,a0		x=x-1
	subq.w	#1,d0
	bpl.b	.nx_yloop
	rts

	CNOP	0,4
.nx_walkx	move.w	d2,d0		numpixels (deltax)
	neg.w	d2
	add.w	d3,d3		2*deltay
	move.w	d2,d1		d1=(2*deltay-deltax) -2*deltay = -deltax
	addq.l	#1,a0		! coz of pre-decrement
	add.w	d2,d2		d2=2*(deltay-deltax) -2*deltay = -2*deltax

	mcnop4
.nx_xloop	move.b	d4,-(a0)	x=x-1, put pixel
	add.w	d3,d1		d=d+2*deltay
	dbpl	d0,.nx_xloop
	add.l	d5,a0		y=y+sy
	add.w	d2,d1		d=d+2*(deltay-deltax)-2*deltay
	subq.w	#1,d0
	bpl.b	.nx_xloop
	rts


	CNOP	0,4
.pos_dx	cmp.w	d3,d2
	bhs.b	.px_walkx

	move.w	d3,d0		numpixels (deltay)
	neg.w	d3
	add.w	d2,d2		2*deltax
	move.w	d3,d1		d1=(2*deltax-deltay) -2*deltax = -deltay
	add.w	d3,d3		d3=2*(deltax-deltay) -2*deltax = -2*deltay

	mcnop4
.px_yloop	move.b	d4,(a0)		put pixel
	add.w	d2,d1		d=d+2*deltax
	add.l	d5,a0		y=y+sy
	dbpl	d0,.px_yloop
	add.w	d3,d1		d=d+2*(deltax-deltay)-2*deltax
	addq.l	#1,a0		x=x+1
	subq.w	#1,d0
	bpl.b	.px_yloop
	rts

	CNOP	0,4
.px_walkx	move.w	d2,d0		numpixels (deltax)
	neg.w	d2
	add.w	d3,d3		2*deltay
	move.w	d2,d1		d1=(2*deltay-deltax) -2*deltay = -deltax
	add.w	d2,d2		d2=2*(deltay-deltax) -2*deltay = -2*deltax

	mcnop4
.px_xloop	move.b	d4,(a0)+	put pixel, x=x+1
	add.w	d3,d1		d=d+2*deltay
	dbpl	d0,.px_xloop
	add.l	d5,a0		y=y+sy
	add.w	d2,d1		d=d+2*(deltay-deltax)-2*deltay
	subq.w	#1,d0
	bpl.b	.px_xloop
d
	rts