;
	
*********************************************************************
*
* Your line routine.
* Every time you are given: A0.L : Chunky Screen (256x256x8)
*
* d0.w/d1.w  x1/y1
* d2.w/d3.w  x2/y2
* d4.b       color
*
* other registers are in undefined state !!
* you may trash all regs except a7
*
*********************************************************************

_line
c

;****************************************************************************
;**
;*       Size-optimized chunky linedraw routine, B
;*
;*  This is one of my contributions to #amycoders linedrawing-compo. This
;*  routine will participate in size-optimizing competition. This code works
;*  with all 0x0 processors, with any cache setup.
;*
;*  This is a simple Bresenham's line algorithm implementation.
;*
;*  Code size without rts: 64 bytes.
;*  Average time (dec): 45 (030@59.220MHz)
;*
;*  Coded by Harry "Piru" Sintonen <sintonen@jyu.fi>.
;*  This code is copyright  1997 by Harry Sintonen, and was written 13th-
;*  19th september 1997.
;*
;****************************************************************************
;*

;  IN: d0.w=x1, d1.w=y1, d2.w=x2, d3.w=y2, d4.b=color
;      a0=chunky buffer (256x256x8)
; OUT: line :) and d0-d3/d5-d6/a0 trashed

S_WIDTH2	EQU	8
S_WIDTH	EQU	1<<S_WIDTH2		256 (screen width)

	moveq	#1,d5		2	sx=1
	sub.w	d0,d2		2	d2=deltax
	bpl.b	.noxswap	2
	neg.w	d2		2
	moveq	#-1,d5		2	sx=-1
.noxswap
	moveq	#S_WIDTH>>6,d6	2
	lsl.l	#6,d6		2	sy=S_WIDTH
	sub.w	d1,d3		2	d3=deltay
	bpl.b	.noyswap	2
	neg.w	d3		2
	neg.l	d6		2	sy=-S_WIDTH
.noyswap
	ext.l	d1		2
	lsl.l	#S_WIDTH2,d1	2
	or.w	d0,d1		2

	cmp.w	d3,d2		2
	add.l	d1,a0		2	add x1+y1*S_WIDTH to chunky screen
	bhs.b	.x		2
	exg	d2,d3		2	swap deltax & deltay
	exg	d5,d6		2	swap sx & sy
.x
	move.w	d2,d0		2	numpixels (deltax)
	neg.w	d2		2
	add.w	d3,d3		2	2*deltay
	move.w	d2,d1		2	d1=(2*deltay-deltax) -2*deltay = -deltax
	add.w	d2,d2		2	d2=2*(deltay-deltax) -2*deltay = -2*deltax

.loop	move.b	d4,(a0)		2	put pixel
	add.w	d3,d1		2	d=d+2*deltay
	add.l	d5,a0		2	x=x+sx
	bmi.b	.dbf		2

	add.w	d2,d1		2	d=d+2*(deltay-deltax)-2*deltay
	add.l	d6,a0		2	y=y+sy
.dbf	dbf	d0,.loop	4

d
	rts